;; Copyright 2009, 2019 Kevin Ryde.

;; This file is part of tcd-format.
;;
;; tcd-format is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; tcd-format is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with tcd-format; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Generate autoloads for emacsen-startup (which is 50tcd-format.el).

(require 'autoload)

(defadvice make-autoload (after my-zap-docstring disable)
  "Zap docstrings in `autoload' forms per `my-zap-autoload-form'."
  (setq ad-return-value (my-zap-autoload-form ad-return-value)))

(defun my-zap-autoload-form (form)
  "If FORM is an (autoload ...) return it with docstring zapped.
Otherwise return FORM unchanged."

  (setq form (copy-sequence form))

  ;;  0        1     2      3           4                5
  ;; (autoload 'func "file" "docstring" interactive-flag type)

  ;; filename no path
  (when (and (eq 'autoload (car-safe form))
             (consp (nth 1 form))
             (stringp (nth 2 form)))
    (setcar (nthcdr 2 form)
            (file-name-nondirectory (nth 2 form))))

  ;; nil for docstring
  (when (and (eq 'autoload (car-safe form))
             (consp (nth 1 form))
             (stringp (nth 3 form)))
    (setcar (nthcdr 3 form)
            nil))

  ;; drop trailing nil docstring
  (my-list-remove-trailing-nils form))

;; (my-zap-autoload-form nil)
;; (my-zap-autoload-form '(autoload 'func "../file" "docstring" interactive-flag))
;; (my-zap-autoload-form '(autoload 'func "../../file" "docstring" nil nil))
;; (my-zap-autoload-form '(autoload 'func "file"))

(defun my-list-remove-trailing-nils (lst)
  "Remove trailing nil elements from list LST."
  (setq lst (reverse lst))
  (while (and (consp lst) (eq nil (car lst)))
    (setq lst (cdr lst)))
  (reverse lst))
;; (my-list-remove-trailing-nils '(1 3 nil nil))
;; (my-list-remove-trailing-nils '(nil nil))

(defun my-ad-advice-enabled-p (func class name)
  "Return non-nil if FUNC advice CLASS and NAME is enabled."
  (let ((advice (ad-find-advice func class name)))
    (and advice
         (ad-advice-enabled advice))))

(defmacro my-with-ad-enable-advice (func class name &rest body)
  "Evaluate BODY with FUNC advice CLASS and NAME temporarily enabled.
And `unwind-protect' ensures the advice is re-disabled (if it was
disabled to start with) no matter what BODY does.

In the current implementation CLASS must be a specific class,
like `before' or `around', it can't be `any'."

  (let ((oldsym (make-symbol "old-enable")))
    `(let ((,oldsym (my-ad-advice-enabled-p ,func ,class ,name)))
       (unwind-protect
           (progn
             (unless ,oldsym
               (ad-enable-advice ,func ,class ,name)
               (ad-activate ,func))
             ,@body)
         (unless ,oldsym
           (ad-disable-advice ,func ,class ,name)
           (ad-activate ,func))))))
(put 'my-with-ad-enable-advice 'lisp-indent-function 3)

;;
;; (progn (ad-enable-advice 'make-autoload 'after 'my-zap-docstring) (ad-activate 'make-autoload))
;; (progn (ad-disable-advice 'make-autoload 'after 'my-zap-docstring) (ad-activate 'make-autoload))
;; (make-autoload '(defun foo () "docstring" 123) "foo.el")
;; (my-with-ad-enable-advice 'make-autoload 'after 'my-zap-docstring (make-autoload '(defun foo () "docstring" 123) "foo.el"))
;;
;; (ad-find-advice 'make-autoload 'after 'my-zap-docstring)
;; (my-ad-advice-enabled-p 'make-autoload 'after 'my-zap-docstring)
;;
;; (my-with-ad-enable-advice 'make-autoload 'after 'my-zap-docstring (my-ad-advice-enabled-p 'make-autoload 'after 'my-zap-docstring))
;;

(defun my-zap-blank-lines ()
  "Zap blank lines between autoload forms."
  (save-excursion
    (while (search-forward ")\n\n(" nil t)
      (replace-match ")\n(" t t))))

(defun my-generate-autoloads (filename)
  "Like `generate-autoloads' but don't bother with docstrings."
  (let ((print-quoted t))
    (my-with-ad-enable-advice 'make-autoload 'after 'my-zap-docstring
      (save-excursion
        (generate-file-autoloads filename))
      (my-zap-blank-lines))))

(my-generate-autoloads "../tcd-format.el")
